#pragma once

#include "BitStream.h"

class DeflateBlock
{
public:
    DeflateBlock(BitStream* inputStream, BitStream* outputStream);

    void readHeader();

    void readDynamicHuffmanTable();

    void buildCodeLengthMapping();

    void readLiteralCodeLengths();

    bool readNextCodeLengthSymbol(unsigned char& buffer);

    bool isFinalBlock() const
    {
        return mInFinalBlock;
    }

    void setCodeLengthAlphabetLengths(const std::vector<unsigned char>& lengths);

    void setCodeLengthLength(unsigned length);

    void setLiteralsTableLength(unsigned length);

    void setDistanceTableLength(unsigned length);

    void setIsFinalBlock(bool isFinal);

    void flushToStream();

private:
    BitStream* mInputStream;
    BitStream* mOutputStream;

    unsigned mHlit{0};
    unsigned mHdist{0};
    unsigned mHclen{0};

    using CodeLengthEntry = std::pair<unsigned char, unsigned>;
    using CodeLengthCountEntry = std::pair<unsigned, std::vector<CodeLengthEntry> >;
    std::vector<CodeLengthCountEntry> mCodeLengthMapping;

    std::vector<unsigned char> mCodeLengthAlphabetLengths;
    static constexpr unsigned CODE_LENGTH_ALPHABET_PERMUTATION[19]{16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15};

    unsigned char mCompressionType{0};

    bool mInFinalBlock{false};

    static constexpr unsigned char NO_COMPRESSION = 0x00;
    static constexpr unsigned char FIXED_HUFFMAN = 0x01;
    static constexpr unsigned char DYNAMIC_HUFFMAN = 0x02;
    static constexpr unsigned char ERROR = 0x03;
};
