#pragma once

#include "ByteUtils.h"
#include "BitStream.h"

#include <vector>
#include <iostream>

class ZlibData
{
public:
    void setByte(unsigned idx, unsigned char data)
    {
        mBitStream.setByte(idx, data);
    }

    void setDataSize(std::size_t size)
    {
        mBitStream.setBufferSize(size);
    }

    void setCompressionMethod(unsigned char method)
    {
        std::cout << "Got compression input " << static_cast<int>(method) << std::endl;
        mCmf = method;
        mCompressionMethod = ByteUtils::getLowerNBits(method, 4);
        mCompressionInfo = ByteUtils::getHigherNBits(method, 4);

        std::cout << "Got compression method " << static_cast<int>(mCompressionMethod) << " and info " << static_cast<int>(mCompressionInfo) << std::endl;
    }

    void setExtraFlags(unsigned char extraFlags)
    {
        std::cout << "Got flags  " << static_cast<int>(extraFlags) << std::endl;

        mFlg = extraFlags;
        mFlagCheck = ByteUtils::getLowerNBits(extraFlags, 5);
        mFlagDict = ByteUtils::getBitN(extraFlags, 5);
        mFlagLevel = ByteUtils::getHigherNBits(extraFlags, 2);

        std::cout << "Got flag check " << static_cast<int>(mFlagCheck) << " and dict " << static_cast<int>(mFlagDict) << " and level " << static_cast<int>(mFlagLevel) << std::endl;
    }

    void processData()
    {
        unsigned char NO_COMPRESSION = 0x00;
        unsigned char FIXED_HUFFMAN = 0x01;
        unsigned char DYNAMIC_HUFFMAN = 0x02;
        unsigned char ERROR = 0x03;

        bool in_final_block = false;
        while(mBitStream.loadNextByte())
        {
            auto working_byte = mBitStream.getCurrentByte();
            std::cout << "Into process data, byte is: " << static_cast<unsigned>(working_byte) << std::endl;

            unsigned char final_block{0};
            mBitStream.getNextNBits(1, final_block);
            if (final_block)
            {
                std::cout << "Got final block" << std::endl;
                in_final_block = true;
            }

            unsigned char compress_type{0};
            mBitStream.getNextNBits(2, compress_type);
            std::cout << "Compress type byte is: " << static_cast<unsigned>(compress_type) << std::endl;
            if (compress_type == NO_COMPRESSION)
            {
                std::cout << "Got NO_COMPRESSION" << std::endl;
            }
            else if (compress_type == FIXED_HUFFMAN)
            {
                std::cout << "Got FIXED_HUFFMAN" << std::endl;
            }
            else if (compress_type == DYNAMIC_HUFFMAN)
            {
                std::cout << "Got DYNAMIC_HUFFMAN" << std::endl;

                unsigned char h_list{0};
                mBitStream.getNextNBits(5, h_list);
                mHlist = h_list + 257;
                std::cout << "Got HLIST " << mHlist << std::endl;

            }
            else if (compress_type == ERROR)
            {
                std::cout << "Got ERROR" << std::endl;
            }
            break;
        }
    }

private:
    BitStream mBitStream;

    unsigned mHlist{0};

    unsigned char mCmf{0};
    unsigned char mFlg{0};
    unsigned char mCompressionMethod{0};
    unsigned char mCompressionInfo{0};
    unsigned char mFlagCheck{0};
    unsigned char mFlagDict{0};
    unsigned char mFlagLevel{0};
    unsigned char mCheckValue{0};
};
