#include "ZlibEncoder.h"

#include "ByteUtils.h"
#include "DeflateEncoder.h"
#include "FileLogger.h"

#include <iostream>

ZlibEncoder::ZlibEncoder(BitStream* inputStream, BitStream* outputStream)
: AbstractEncoder(inputStream, outputStream)
{

}

ZlibEncoder::~ZlibEncoder()
{

}

void ZlibEncoder::setCompressionMethod(unsigned char method)
{
    std::cout << "Got compression input " << static_cast<int>(method) << std::endl;
    mCmf = method;
    mCompressionMethod = ByteUtils::getLowerNBits(method, 4);
    mCompressionInfo = ByteUtils::getHigherNBits(method, 4);

    std::cout << "Got compression method " << static_cast<int>(mCompressionMethod) << " and info " << static_cast<int>(mCompressionInfo) << std::endl;
}

void ZlibEncoder::setExtraFlags(unsigned char extraFlags)
{
    std::cout << "Got flags  " << static_cast<int>(extraFlags) << std::endl;

    mFlg = extraFlags;
    mFlagCheck = ByteUtils::getLowerNBits(extraFlags, 5);
    mFlagDict = ByteUtils::getBitN(extraFlags, 5);
    mFlagLevel = ByteUtils::getHigherNBits(extraFlags, 2);

    std::cout << "Got flag check " << static_cast<int>(mFlagCheck) << " and dict " << static_cast<int>(mFlagDict) << " and level " << static_cast<int>(mFlagLevel) << std::endl;
}

bool ZlibEncoder::encode()
{
    if (!mWorkingEncoder)
    {
        if (mCompressionMethod == 8)
        {
            mWorkingEncoder = std::make_unique<DeflateEncoder>(mInputStream, mOutputStream);
        }
        else
        {
            MLOG_ERROR("Zib requested decoder not recognized: " << mCompressionMethod << " aborting encode");
            return false;
        }
    }
    return mWorkingEncoder->encode();
}

bool ZlibEncoder::decode()
{
    if (!mWorkingEncoder)
    {
        if (mCompressionMethod == 8)
        {
            mWorkingEncoder = std::make_unique<DeflateEncoder>(mInputStream, mOutputStream);
        }
        else
        {
            MLOG_ERROR("Zib requested decoder not recognized: " << mCompressionMethod << " aborting decode");
            return false;
        }
    }
    return mWorkingEncoder->decode();
}
