#pragma once

#include "File.h"
#include "Image.h"
#include "PngElements.h"
#include "ZlibEncoder.h"

#include <string>
#include <memory>
#include <filesystem>

class BitStream;
class ZlibEncoder;

using Path = std::filesystem::path;

class PngReader
{
public:
    ~PngReader();
    void setPath(const Path& path);

    std::unique_ptr<Image<unsigned char> > read();
private:
    bool readChunk();

    void readHeaderChunk();

    void readIDATChunk(unsigned length);

    void logHeader();

    bool checkSignature();

    unsigned mCurrentOffset{0};

    Png::IHDRChunk mIHDRChunk;

    std::unique_ptr<Image<unsigned char> > mWorkingImage;
    std::unique_ptr<File> mFile;
    Path mPath;

    std::unique_ptr<ZlibEncoder> mEncoder;
    std::unique_ptr<BitStream> mInputStream;
    std::unique_ptr<BitStream> mOutputStream;
    bool mAwaitingDataBlock{true};
};
